﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Classification.kNN;

namespace Classification.NN
{
	public enum ZeroNeighborsStrategy
	{
		TakeClosest
	}

	public class NP<T, U>
		where T : INeighborable
		where U : IClassifiable
	{
		public static string Classify(IEnumerable<T> trainingSet, U item, double range, ZeroNeighborsStrategy zns = ZeroNeighborsStrategy.TakeClosest)
		{
			foreach (var t in trainingSet)
			{
				t.CalculateDistance(item);
			}

			var neighbors = trainingSet.Where(x => x.GetDistance() <= range).ToList();

			if (!neighbors.Any())
			{
				switch (zns)
				{
					case ZeroNeighborsStrategy.TakeClosest:
						neighbors = trainingSet.Where(x => x.GetDistance() == trainingSet.Min(y => y.GetDistance())).ToList();
						break;
					default:
						break;
				}
			}

			Dictionary<string, double> elections = PerformVoting(neighbors);

			string bestClass = SelectBestClass(elections);

			item.SetClass(bestClass);

			return bestClass;
		}

		private static Dictionary<string, double> PerformVoting(List<T> neighbors)
		{
			Dictionary<string, double> elections = new Dictionary<string, double>();

			foreach (var neighbor in neighbors)
			{
				if (!elections.ContainsKey(neighbor.GetClass()))
				{
					elections.Add(neighbor.GetClass(), neighbor.GetWeight());
				}
				else
				{
					elections[neighbor.GetClass()] += neighbor.GetWeight();
				}
			}

			return elections;
		}

		private static string SelectBestClass(Dictionary<string, double> elections)
		{
			double bestResult = -1;
			string bestClass = null;
			foreach (var vote in elections)
			{
				if (vote.Value > bestResult)
				{
					bestClass = vote.Key;
					bestResult = vote.Value;
				}
			}
			return bestClass;
		}
	}
}
