﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Classification.kNN
{
	public enum EquidistantVotesTreatment
	{
		Random,
		Confidence,
		TakeAll
	}

	public class kNP<T, U>
		where T : INeighborable
		where U : IClassifiable
	{
		public static string Classify(List<T> trainingSet, U item, int k, EquidistantVotesTreatment evt)
		{
			List<T> neighbors = GetOrderedNeighbors(trainingSet, item, k);

			Dictionary<string, double> elections = PerformVoting(neighbors, k, evt);

			string bestClass = SelectBestClass(elections);

			item.SetClass(bestClass);

			return bestClass;
		}

		private static List<T> GetOrderedNeighbors(List<T> trainingSet, U item, int k)
		{
			var neighbors = new List<T>();

			for (int i = 0; i < k; i++)
			{
				trainingSet[i].CalculateDistance(item);
				for (int j = 0; j < neighbors.Count; j++)
				{
					if (trainingSet[i].GetDistance() <= neighbors[j].GetDistance())
					{
						neighbors.Insert(j, trainingSet[i]);
						break;
					}
				}
				if (neighbors.Count == i)
				{
					neighbors.Add(trainingSet[i]);
				}
			}

			for (int i = k; i < trainingSet.Count; i++)
			{
				trainingSet[i].CalculateDistance(item);
				for (int j = 0; j < k; j++)
				{
					if (trainingSet[i].GetDistance() <= neighbors[j].GetDistance())
					{
						neighbors.Insert(j, trainingSet[i]);
						break;
					}
				}
			}

			return neighbors;
		}

		private static Dictionary<string, double> PerformVoting(List<T> neighbors, int k, EquidistantVotesTreatment evt)
		{
			Dictionary<string, double> elections = new Dictionary<string, double>();

			if (neighbors.Count > k && neighbors[k - 1].GetDistance() == neighbors[k].GetDistance())
			{
				switch (evt)
				{
					case EquidistantVotesTreatment.TakeAll:
						for (int i = 0; i < neighbors.Count && (i < k || neighbors[i].GetDistance() == neighbors[i - 1].GetDistance()); i++)
						{
							Vote(elections, neighbors[i]);
						}
						break;

					case EquidistantVotesTreatment.Confidence:
						int numberOfVotes = 0;
						double theDistance = neighbors[k].GetDistance();
						while (neighbors[0].GetDistance() < theDistance)
						{
							Vote(elections, neighbors[0]);
							neighbors.RemoveAt(0);
							numberOfVotes++;
						}

						var equidistantNeighbors = neighbors.Where(x => x.GetDistance() == theDistance);

						while (numberOfVotes < k)
						{
							double bestConfidence = 0;
							T bestNeighbor = default(T);
							foreach (var en in equidistantNeighbors)
							{
								if (en.GetConfidence() > bestConfidence)
								{
									bestConfidence = en.GetConfidence();
									bestNeighbor = en;
								}
							}

							Vote(elections, bestNeighbor);
							numberOfVotes++;
						}
						break;

					case EquidistantVotesTreatment.Random:
						for (int i = 0; i < k && i < neighbors.Count; i++)
						{
							Vote(elections, neighbors[i]);
						}
						break;
				}
			}
			else
			{
				for (int i = 0; i < k && i < neighbors.Count; i++)
				{
					Vote(elections, neighbors[i]);
				}
			}

			return elections;
		}

		private static void Vote(Dictionary<string, double> elections, T neighbor)
		{
			if (!elections.ContainsKey(neighbor.GetClass()))
			{
				elections.Add(neighbor.GetClass(), neighbor.GetWeight());
			}
			else
			{
				elections[neighbor.GetClass()] += neighbor.GetWeight();
			}
		}

		private static string SelectBestClass(Dictionary<string, double> elections)
		{
			double bestResult = -1;
			string bestClass = null;
			foreach (var vote in elections)
			{
				if (vote.Value > bestResult)
				{
					bestClass = vote.Key;
					bestResult = vote.Value;
				}
			}
			return bestClass;
		}
	}
}
