// This is the main DLL file.

#include "stdafx.h"

#include "TM.h"
#include "CMRmisc.h"
#include "TextTree.h"
#include "PatternTree.h"
#include "OccList.h"
#include "OccLongList.h"

using namespace TM;

int MIN_VERTEX;
int MAX_VERTEX;

int currentVertexNumber;
PatternTree currentPatternTree;

int** TreeMiner::Process(int** documents, int support)
{
	vector<vector<int>> result;

	MIN_VERTEX = INT_MAX;
	MAX_VERTEX = -1;

	vector<int> frequency(1000,0); //assuming the max frequent tree size is 1000
	vector<int> checked(1000,0);
	vector<int> closed(1000,0);
	vector<int> maximal(1000,0);

	currentPatternTree.initialSize();

	vector<TextTree> database;

	for(int l = 0; l < documents[0][0]; l++)
	{
		TextTree tt;
		tt = &documents[l + 1][0];
		tt.tid = l;
		for ( int i = 0; i < tt.vNumber; i++ )
		{
			if ( tt.vLabel[i] < MIN_VERTEX ) MIN_VERTEX = tt.vLabel[i];
			if ( tt.vLabel[i] > MAX_VERTEX ) MAX_VERTEX = tt.vLabel[i];
		}
		database.push_back(tt);
	}

	/******************************************************************
	step2.1: scan the database once, find frequent node labels
	******************************************************************/
	vector<bool> isFrequent(MAX_VERTEX - MIN_VERTEX + 1, false);
	map<int,int> count;
	map<int,int>::iterator pos;

	for ( int i = 0; i < database.size(); i++ ) {
		vector<bool> isVisited(MAX_VERTEX - MIN_VERTEX + 1, false);
		for ( int j = 0; j < database[i].vNumber; j++ ) {
			int temp = database[i].vLabel[j] - MIN_VERTEX;
			if ( !isVisited[temp] ) {
				isVisited[temp] = true;
				pos = count.find(temp);
				if ( pos != count.end() )
					count[temp]++;
				else
					count.insert(make_pair(temp, 1));
			}
		}
	}
	for ( int i = 0; i < isFrequent.size(); i++ ) {
		if ( count[i] >= support ) isFrequent[i] = true;
	}

	/******************************************************************
	step2.2: scan the database another time, to get occurrenceList for all 
	frequent nodes 
	******************************************************************/
	map<int,OccLongList> occLongList;
	map<int,OccLongList>::iterator pos2;
	vector<int> dummy;
	for ( int i = 0; i < database.size(); i++ ) {
		for ( int j = 0; j < database[i].vNumber; j++ ) {
			if ( isFrequent[database[i].vLabel[j] - MIN_VERTEX] == true ) {
				occLongList[database[i].vLabel[j] - MIN_VERTEX].insert(i,dummy,j);
			}
		}
	}

	/******************************************************************
	step2.3: explore each frequent item 
	******************************************************************/
	for ( pos2 = occLongList.begin(); pos2 != occLongList.end(); ++pos2 ) {
		if ( pos2->second.mySupport >= support ) {
			currentPatternTree.addRightmost(pos2->first + MIN_VERTEX,0);
			pos2->second.explore(isFrequent,database,support,checked,closed,maximal,result);
			currentPatternTree.deleteRightmost();
		}
	}

	int** toReturn = new int*[result.size() + 1];
	toReturn[0] = new int[1];
	toReturn[0][0] = result.size();
	for(int i = 0; i < result.size(); i++)
	{
		toReturn[i+1] = new int[result[i].size() + 1];
		toReturn[i+1][0] = result[i].size();
		for(int j = 0; j < result[i].size(); j++)
		{
			toReturn[i+1][j+1] = result[i][j];
		}
	}

	return toReturn;
}