/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
Struct: TextTree

Description: Used to store a rooted ordered tree, i.e., 
a transaction in the database.
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "stdafx.h"
#include "CMRmisc.h"
#include "TextTree.h"

void TextTree::operator=(int* in)
{
	int total;
	int temp;
	stack<int> tempK;

	this->tid = in[0];
	this->tid = in[1]; //read tid twice

	total = in[2];
	temp = in[3]; //read in the root label

	this->vLabel.push_back(temp);
	this->firstChild.push_back(-1); //temporarily, the root has no child
	this->nextSibling.push_back(-1); //the root has no sibling
	this->parent.push_back(-1); //the root has no parent
	this->vNumber = 1;

	tempK.push(0); //the index of the root

	for ( int i = 1; i < total; i++ )
	{
		temp = in[i+3];
		if ( temp == -1 ) //a backtrack
		{
			tempK.pop();
			continue; //nothing to do with the TextTree
		}

		this->vLabel.push_back(temp); //add the new vertex label

		if (this->firstChild[tempK.top()] == -1) //if the current node has no child yet
		{
			this->firstChild[tempK.top()] = this->vNumber;	
		}
		else //if the current node has children already, find the rightmost child, its nextSibling is the new node
		{
			int j = tempK.top();
			j = this->firstChild[j];
			while ( this->nextSibling[j] != -1 )
			{
				j = this->nextSibling[j];
			}
			this->nextSibling[j] = this->vNumber;
		}

		this->firstChild.push_back(-1); //the new node has no child yet
		this->nextSibling.push_back(-1); //the new node has no right sibling yet
		this->parent.push_back(tempK.top()); //the parent of the new node is the current node
		tempK.push(this->vNumber);
		this->vNumber++;
	}
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
dfsVisit(const int, const TextTree& rhs, vector<int>& zakiCode)

Decription: a helper function for depth-first-visit
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void dfsVisit(const int current, const TextTree& rhs, vector<int>& zakiCode)
{
	zakiCode.push_back(rhs.vLabel[current]);
	int i = rhs.firstChild[current];
	while ( i != -1 )
	{
		dfsVisit(i,rhs,zakiCode);
		i = rhs.nextSibling[i];
	}
	zakiCode.push_back(-1);
}

void operator<<(vector<int>& out, const TextTree& rhs)
{
	dfsVisit(0, rhs, out);
}
