﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Classification.kNN;
using Plossum.CommandLine;
using XmlClassifier.Algorithms;
using XmlClassifier.DataStructures.Patterns;

namespace XmlClassifier
{
	public enum Algorithm
	{
		NP,
		NN,
		KNP,
		KNN,
		Matching,
		Rules
	}

	public enum Pattern
	{
		Tree,
		Path
	}

	[CommandLineManager(ApplicationName = "XClassifier", Copyright = "Copyright (c) Maciej Piernik")]
	public class Options
	{
		#region General

		[CommandLineOption(Description = "Displays this help text")]
		public bool Help = false;

		[CommandLineOption(Description = "Display author information.")]
		public bool NoInfo = false;

		[CommandLineOption(Description = "Training dataset", MinOccurs = 1)]
		public string Train;

		[CommandLineOption(Description = "Test dataset", MinOccurs = 1)]
		public string Test;

		[CommandLineOption(Description = "Classification algorithm")]
		public Algorithm Algorithm = Algorithm.NP;

		[CommandLineOption(Description = "Evaluation method")]
		public Evaluation.CostModel EvaluationMethod = Evaluation.CostModel.Percent;

		#endregion

		#region Neighborhood

		[CommandLineOption(Description = "Number of neighbors (0 = all)")]
		public int K = 1;

		[CommandLineOption(Description = "Range in which objects are regarded as neighbors (0-1)")]
		public double Range = 0.2;

		#endregion

		#region Patterns

		[CommandLineOption(Description = "If you don't want to use cached patterns from previous runs.")]
		public bool NoCache = false;

		[CommandLineOption(Description = "Minimal support parameter")]
		public double MinSup = 0.01;

		[CommandLineOption(Description = "Pattern definition")]
		public Pattern Pattern = Pattern.Tree;

		[CommandLineOption(Description = "Use maximal paths only")]
		public bool MaxPaths = false;

		[CommandLineOption(Description = "Maximal length of paths (0 = unlimited)")]
		public int MaxPathLength = 0;

		[CommandLineOption(Description = "Duplicate pattern treatment")]
		public TreePattern.Duplicates DuplicateTreatment = TreePattern.Duplicates.RemoveEqual;

		#endregion

		#region Voting

		[CommandLineOption(Description = "Weight votes by distance.")]
		public bool WeightDistance = false;

		[CommandLineOption(Description = "Weight votes by support.")]
		public bool WeightSupport = false;

		[CommandLineOption(Description = "Weight votes by confidence.")]
		public bool WeightConfidence = false;

		[CommandLineOption(Description = "Weight votes by pattern size.")]
		public bool WeightSize = false;

		[CommandLineOption(Description = "Use more than k nearest neighbors if there are more neighbors at the same distance.")]
		public EquidistantVotesTreatment EquidistantVotes = EquidistantVotesTreatment.TakeAll;

		#endregion
	}
}
