﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using XmlClassifier.DataStructures;
using XmlClassifier.DataStructures.Patterns;
using XmlClassifier.Helpers;

namespace XmlClassifier.Algorithms.Classification
{
	public class Simple
	{
		public static void Classify(List<BasePattern> patterns, ref List<Document> trainingSet)
		{
			int numberOfClasses = patterns.Select(x => x.Class).Distinct().Count();
			int[] classCounts = new int[numberOfClasses];
			double[] tmp = new double[numberOfClasses];
			for (int i = 0; i < numberOfClasses; i++)
			{
				classCounts[i] = patterns.Count(x => x.Class == i);
			}

			foreach (var document in trainingSet)
			{
				DebugConsole.SetCursorPosition(0, Console.CursorTop);
				DebugConsole.Write(string.Format("{0,3:0}%", (double)(trainingSet.IndexOf(document)) * 100 / trainingSet.Count));

				for (int i = 0; i < numberOfClasses; i++)
				{
					tmp[i] = 0;
				}

				foreach (var pattern in patterns)
				{
					if (pattern.Matches(document))
					{
						tmp[pattern.Class]++;
					}
				}

				double maxVal = 0;
				double currVal = 0;
				int bestClass = -1;
				for (int i = 0; i < numberOfClasses; i++)
				{
					currVal = tmp[i] / (double)classCounts[i];
					if (currVal > maxVal)
					{
						maxVal = currVal;
						bestClass = i;
					}
				}

				document.ActualClass = bestClass;
			}

			DebugConsole.WriteLine("");
		}
	}
}
