﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace XmlClassifier.Algorithms.DistanceMeasures.PTED
{
	/// <summary>
	/// A new version of partial tree-edit distance
	/// </summary>
	public static partial class PTED
	{
		/// <summary>
		/// Partial tree-edit distance algorithm.
		/// The function corresponds to the Delta function described in the article.
		/// </summary>
		/// <param name="P">Pattern tree</param>
		/// <param name="T">Document tree</param>
		/// <returns>How much does P need to be modified to become a subtree of T?</returns>
		public static int Original(XElement P, XElement T)
		{
			int tmp = delta(new List<XElement>() { P }, new List<XElement>() { T });
			int pted = tmp;

			foreach (var u in T.Descendants())
			{
				tmp = delta(new List<XElement>() { P }, new List<XElement>() { u });
				if (tmp < pted)
				{
					pted = tmp;
				}
			}

			foreach (var v in P.Descendants())
			{
				tmp = delta(new List<XElement>() { v }, new List<XElement>() { T }) + (P.Descendants().Count() - v.Descendants().Count());
				if (tmp < pted)
				{
					pted = tmp;
				}
			}

			return pted;
		}

		private static int delta(List<XElement> Fv, List<XElement> Fw)
		{
			int result = 0;

			if (!Fv.Any())
			{
				result = 0;
			}
			else if (!Fw.Any())
			{
				result = Fv.Sum(x => x.Descendants().Count() + 1);
			}
			else
			{
				int r = delta(Fv, Fw.GetRange(0, Fw.Count - 1));
				int l = delta(Fv.GetRange(0, Fv.Count - 1), Fw) + delta(Fv.Last().Descendants().ToList(), new List<XElement>()) + 1;
				int b = delta(Fv.GetRange(0, Fv.Count - 1), Fw.GetRange(0, Fw.Count - 1)) + delta(Fv.Last().Descendants().ToList(), Fw.Last().Descendants().ToList()) + (Fv.Last().Name.LocalName != Fw.Last().Name.LocalName ? 1 : 0);
				result = Math.Min(Math.Min(r, l), b);
			}

			return result;
		}
	}
}
