﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using XmlClassifier.DataStructures;

namespace XmlClassifier.Algorithms
{
	public class Evaluation
	{
		public enum CostModel
		{
			Percent,
			Equal,
			Proportional,
			Inverse
		}

		public static double Accuracy(List<Document> testingSet, CostModel costModel = CostModel.Equal)
		{
			double result = 0;

			switch (costModel)
			{
				case CostModel.Percent:
					result = testingSet.Count(x => x.ActualClass != -1 && x.ActualClass == x.ExpectedClass) / (double)testingSet.Count(x => x.ActualClass != -1);
					break;

				case CostModel.Equal:
					IEnumerable<int> classes = testingSet.Select(x => x.ExpectedClass).Distinct();
					double weight = (double)1 / (double)classes.Count();
					foreach (var c in classes)
					{
						double accuracy = testingSet.Count(x => x.ActualClass != -1 && x.ExpectedClass == c && x.ActualClass == x.ExpectedClass) / (double)testingSet.Count(x => x.ActualClass != -1 && x.ExpectedClass == c);
						result += weight * accuracy;
					}
					break;

				case CostModel.Proportional:
					classes = testingSet.Select(x => x.ExpectedClass).Distinct();
					foreach (var c in classes)
					{
						weight = (double)testingSet.Count(x => x.ExpectedClass == c) / (double)testingSet.Count;
						double accuracy = testingSet.Count(x => x.ActualClass != -1 && x.ExpectedClass == c && x.ActualClass == x.ExpectedClass) / (double)testingSet.Count(x => x.ActualClass != -1 && x.ExpectedClass == c);
						result += weight * accuracy;
					}
					break;

				case CostModel.Inverse:
					throw new Exception("This cost model is not implemented.");

				default:
					throw new Exception("There's no such cost model!");
			}

			return result;
		}
	}
}
