﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using XmlClassifier.DataStructures;
using XmlClassifier.DataStructures.Patterns;
using XmlClassifier.Helpers;

namespace XmlClassifier.Algorithms
{
	public class FirstkNN
	{
		public static void Test(List<Document> trainingSet, ref List<Document> testSet, int k)
		{
			foreach (var d in testSet)
			{
				DebugConsole.SetCursorPosition(0, Console.CursorTop);
				DebugConsole.Write(string.Format("{0,3:0}%", (double)(testSet.IndexOf(d)) * 100 / testSet.Count));

				var kNearestNeighbors = new List<ComparableDocument>();

				for (int i = 0; i < k; i++)
				{
					int distance = TED.Process(d.XDocument, trainingSet[i].XDocument);
					for (int j = 0; j < kNearestNeighbors.Count; j++)
					{
						if (distance <= kNearestNeighbors[j].Distance)
						{
							kNearestNeighbors.Insert(j, new ComparableDocument() { Distance = distance, Element = trainingSet[i] });
							break;
						}
					}
					if (kNearestNeighbors.Count == i)
					{
						kNearestNeighbors.Add(new ComparableDocument() { Distance = distance, Element = trainingSet[i] });
					}
				}

				for (int i = k; i < trainingSet.Count; i++)
				{
					int distance = TED.Process(d.XDocument, trainingSet[i].XDocument);
					for (int j = 0; j < k; j++)
					{
						if (distance <= kNearestNeighbors[j].Distance)
						{
							kNearestNeighbors.Insert(j, new ComparableDocument() { Distance = distance, Element = trainingSet[i] });
							break;
						}
					}
				}

				int numberOfClasses = trainingSet.Select(x => x.ExpectedClass).Distinct().Count();
				int[] elections = new int[numberOfClasses];

				for (int i = 0; i < kNearestNeighbors.Count && (i < k || kNearestNeighbors[i].Distance == kNearestNeighbors[i - 1].Distance); i++)
				{
					elections[kNearestNeighbors[i].Element.ExpectedClass]++;
				}

				int best = -1;
				for (int i = 0; i < numberOfClasses; i++)
				{
					if (elections[i] > best)
					{
						d.ActualClass = i;
						best = elections[i];
					}
				}
			}

			DebugConsole.WriteLine();
		}

		public class ComparableDocument : IComparable
		{
			public int Distance;
			public Document Element;

			public int CompareTo(object obj)
			{
				if (this.Distance > ((ComparableDocument)obj).Distance)
				{
					return 1;
				}
				else if (this.Distance == ((ComparableDocument)obj).Distance)
				{
					return 0;
				}
				else
				{
					return -1;
				}
			}
		}
	}
}
