﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using XmlClassifier.DataStructures;
using XmlClassifier.DataStructures.Patterns;
using XmlClassifier.Helpers;

namespace XmlClassifier.Algorithms
{
	public class FirstkNP
	{
		public static bool WeightDistance = false;

		public static bool WeightSupport = false;

		public static bool WeightConfidence = false;

		public static bool WeightSize = false;

		public static List<BasePattern> Train<T>(List<Document> trainingSet, double minSup) where T : BasePattern, new()
		{
			T patternFactory = new T();

			return patternFactory.GetPatterns(trainingSet, minSup);
		}

		public static void Test(List<BasePattern> patterns, int k, ref List<Document> testSet)
		{
			foreach (var document in testSet)
			{
				DebugConsole.SetCursorPosition(0, Console.CursorTop);
				DebugConsole.Write(string.Format("{0,3:0}%", (double)(testSet.IndexOf(document)) * 100 / testSet.Count));

				var kNearestPatterns = new List<BasePattern>();

				for (int i = 0; i < k; i++)
				{
					patterns[i].CalculateDistance(document);
					for (int j = 0; j < kNearestPatterns.Count; j++)
					{
						if (patterns[i].Distance <= kNearestPatterns[j].Distance)
						{
							kNearestPatterns.Insert(j, patterns[i]);
							break;
						}
					}
					if (kNearestPatterns.Count == i)
					{
						kNearestPatterns.Add(patterns[i]);
					}
				}

				for (int i = k; i < patterns.Count; i++)
				{
					patterns[i].CalculateDistance(document);
					for (int j = 0; j < k; j++)
					{
						if (patterns[i].Distance <= kNearestPatterns[j].Distance)
						{
							kNearestPatterns.Insert(j, patterns[i]);
							break;
						}
					}
				}

				int numberOfClasses = patterns.Select(x => x.Class).Distinct().Count();
				double[] elections = new double[numberOfClasses];

				for (int i = 0; i < kNearestPatterns.Count && (i < k || kNearestPatterns[i].Distance == kNearestPatterns[i - 1].Distance); i++)
				{
					elections[kNearestPatterns[i].Class] += Weight(kNearestPatterns[i]);
				}

				double best = -1;
				for (int i = 0; i < numberOfClasses; i++)
				{
					if (elections[i] > best)
					{
						document.ActualClass = i;
						best = elections[i];
					}
				}
			}

			DebugConsole.WriteLine();
		}

		private static double Weight(BasePattern p)
		{
			double totalWeight = 1;

			if (FirstkNP.WeightDistance)
			{
				totalWeight *= ((double)1 - p.Distance);
			}
			if (FirstkNP.WeightSupport)
			{
				totalWeight *= (double)p.Support;
			}
			if (FirstkNP.WeightConfidence)
			{
				totalWeight *= (double)p.Confidence;
			}
			if (FirstkNP.WeightSize)
			{
				totalWeight *= (double)p.Size;
			}

			return totalWeight;
		}
	}
}
