﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Classification.kNN;

namespace XmlClassifier.DataStructures.Patterns
{
	[Serializable()]
	public abstract class BasePattern : IComparable, INeighborable
	{
		protected double? distance;

		public int Class { get; set; }

		public double Distance
		{
			get
			{
				if (this.distance.HasValue)
				{
					return this.distance.Value;
				}
				else
				{
					throw new Exception("Niezainicjowana odległość!");
				}
			}
		}

		public double Support { get; set; }

		public double Confidence { get; set; }

		public abstract double Size { get; set; }

		public abstract List<BasePattern> GetPatterns(List<Document> documents, double minsup);

		public abstract bool Matches(Document document);

		public abstract int CountMatches(Document document);

		public abstract double CalculateDistance(Document document);

		public int CompareTo(object obj)
		{
			if (this.distance > (obj as BasePattern).distance)
			{
				return 1;
			}
			else if (this.distance == (obj as BasePattern).distance)
			{
				return 0;
			}
			else
			{
				return -1;
			}
		}

		public static void NormalizeWeights(IEnumerable<BasePattern> patterns)
		{
			double maxSup = 0;
			double maxConf = 0;
			double maxSize = 0;

			foreach (var pattern in patterns)
			{
				maxSup = pattern.Support > maxSup ? pattern.Support : maxSup;
				maxConf = pattern.Confidence > maxConf ? pattern.Confidence : maxConf;
				maxSize = pattern.Size > maxSize ? pattern.Size : maxSize;
			}

			foreach (var pattern in patterns)
			{
				pattern.Support = pattern.Support / maxSup;
				pattern.Confidence = pattern.Confidence / maxConf;
				pattern.Size = pattern.Size / maxSize;
			}
		}

		#region kNP
		public static bool WeightDistance = false;

		public static bool WeightSupport = false;

		public static bool WeightConfidence = false;

		public static bool WeightSize = false;

		public void CalculateDistance(IClassifiable o)
		{
			this.distance = CalculateDistance((Document)o);
		}

		public double GetDistance()
		{
			return this.Distance;
		}

		public double GetConfidence()
		{
			return this.Confidence;
		}

		public string GetClass()
		{
			return this.Class.ToString();
		}

		public double GetWeight()
		{
			double totalWeight = 1;

			if (BasePattern.WeightDistance)
			{
				totalWeight *= ((double)1 - this.Distance);
			}
			if (BasePattern.WeightSupport)
			{
				totalWeight *= (double)this.Support;
			}
			if (BasePattern.WeightConfidence)
			{
				totalWeight *= (double)this.Confidence;
			}
			if (BasePattern.WeightSize)
			{
				totalWeight *= (double)this.Size;
			}

			return totalWeight;
		
		}
		#endregion
	}
}
